/*
   DScope Application

   Copyright (C) 2008,2009:
         Daniel Roggen, droggen@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef __MAINWINDOW_H
#define __MAINWINDOW_H

#include <QMainWindow>
#include <QLabel>
#include <QKeyEvent>
#include <QImage>
#include <QSplitter>
#include <QtNetwork>
#include <QTcpSocket>
#include <vector>
#include "helper.h"
#include "scope/DScopesQT.h"
#include "scope/DScopesQTWidget.h"
#include "scope/Scope.h"

//#include <qextserialport.h>
#include <QtSerialPort/QSerialPort>

#include "FrameParser/FrameParser3.h"
#include "dlinereader.h"



namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:
   virtual void timerEvent(QTimerEvent *event);


private:
   Ui::MainWindow *ui;
   DScopesQTWidget *dscopes;
   int Connected;	// 0: unconnected. 1: tcp. 2: serial
   //QextSerialPort *port;
   QSerialPort *port;
   QTcpSocket *tcpSocket;
   int timer;
   bool binary;
   FrameParser3 fp;
    DLineReader dlr;
   int totreceiveddata;
   int nanvalue;              // Value used to replace NaNs
   unsigned buffersize;     // Size of the buffer to save data / display

   QLabel *datarateLabel;
   QLabel *displayrateLabel;

   double pt;
   double lastplot,refresh;

   double time_received_last,time_received_delta;
   std::vector<double> time_received_delta_vector;

   double time_displayed_last,time_displayed_delta;

   void Plot();
   void receivedData(vector<int> &linedata,std::vector<bool> &linedatanan);
    void receivedQuat(vector<float> &linedataf);
   void treatNaN(vector<int> &linedata,std::vector<bool> &linedatanan);


   // Trace storage
   vector<vector<int> > alldata;
   ScopesDefinition sd;
   vector<unsigned> handles;

   void BinaryChunckDecodeRead(QByteArray &ba);
    void TextChunckDecodeRead(const QByteArray& in);

   void loadConfiguration(QString fileName);
   void SaveData();
   void DeleteAllScopes();
   void AddScopes(unsigned n);


private slots:
   // Autoconnected signal/slot
   void on_actionAbout_triggered();
   void on_actionHowto_triggered();
   void on_actionListSerialPorts_triggered();
   void on_actionSaveConfiguration_triggered();
   void on_actionLoadConfiguration_triggered();
   void on_uipbConnect_clicked();
   void on_uipbApplyBinary_clicked();
   void on_uipbApplyDisplay_clicked();
   void on_pushButton_ClearData_clicked();
   void on_pushButton_SaveData_clicked();
   void on_uiAfterGlow_stateChanged(int state);
   void on_uiRefreshRate_valueChanged(int i);
   void on_uiNanValue_valueChanged(int i);
   void on_uiBufferSize_valueChanged(int i);

   // Programmatically connected signal/slot
   void tcpRead();
   void serialRead();
   void displayError(QAbstractSocket::SocketError);
   void dsrChanged(bool);
   void aboutToClose();
};

#endif // __MAINWINDOW_H
